{
    "$graph": [
        {
            "class": "CommandLineTool",
            "label": "decompress",
            "doc": "Decompress gz input files with gzip\n",
            "hints": [
                {
                    "dockerPull": "mgrast/amplicon:1.0",
                    "class": "DockerRequirement"
                }
            ],
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "stdout": "${\n    if (inputs.output.length){\n      return inputs.output\n    }\n    else{\n       return inputs.file.basename.replace(/\\.gz$/ , '') ;\n    }\n  \n}\n",
            "stderr": "decompress.error",
            "inputs": [
                {
                    "type": "File",
                    "format": [
                        "#decompress.tool.cwl/file/gz"
                    ],
                    "inputBinding": {
                        "position": 3
                    },
                    "id": "#decompress.tool.cwl/file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "",
                    "id": "#decompress.tool.cwl/output"
                }
            ],
            "baseCommand": [
                "gzip"
            ],
            "arguments": [
                "-c",
                "-d"
            ],
            "id": "#decompress.tool.cwl",
            "outputs": [
                {
                    "type": "File",
                    "id": "#decompress.tool.cwl/decompressed",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output.length){\n      return inputs.output\n    }\n    else{\n       return inputs.file.basename.replace(/\\.gz$/ , '') ;\n    }\n  \n}\n"
                    }
                },
                {
                    "type": "File",
                    "id": "#decompress.tool.cwl/error",
                    "outputBinding": {
                        "glob": "decompress.error"
                    }
                }
            ]
        },
        {
            "class": "Workflow",
            "label": "Decompress",
            "doc": "Decompress mate pair fastq files",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "record",
                        "label": "mate_pair",
                        "fields": [
                            {
                                "type": "File",
                                "name": "#main/mate_pair/forward"
                            },
                            {
                                "type": "File",
                                "name": "#main/mate_pair/reverse"
                            }
                        ]
                    },
                    "id": "#main/mate_pair"
                },
                {
                    "type": "string",
                    "default": "0010",
                    "id": "#main/stage"
                }
            ],
            "outputs": [
                {
                    "outputSource": "#main/make_return/mate_pair",
                    "type": "Any",
                    "id": "#main/mate_pair_decompressed"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": [
                        "#main/decompress_forward/decompressed",
                        "#main/decompress_reverse/decompressed"
                    ],
                    "id": "#main/mate_pair_files"
                }
            ],
            "steps": [
                {
                    "label": "STAGE:0010",
                    "run": "#decompress.tool.cwl",
                    "in": [
                        {
                            "source": "#main/mate_pair",
                            "valueFrom": "${ return self.forward }\n",
                            "id": "#main/decompress_forward/file"
                        },
                        {
                            "source": "#main/mate_pair",
                            "valueFrom": "${\n    if (!self.forward.basename.match(/\\.tap\\./) ) {\n        var prefix = self.forward.basename;\n        prefix = prefix.replace( /\\.fastq.gz$/ , '');\n        prefix = prefix.replace(/\\./ , \"_\") ;\n        return prefix + \".tap.\" + \"001\" + \".fastq\";\n    } else if (self.forward.basename.match(/\\.gz$/) ) {\n        return self.forward.basename.replace(/\\.gz$/ , '') ;\n    } else {\n        return self.forward.basename ;\n    }\n }      \n",
                            "id": "#main/decompress_forward/output"
                        }
                    ],
                    "out": [
                        "#main/decompress_forward/decompressed"
                    ],
                    "id": "#main/decompress_forward"
                },
                {
                    "label": "STAGE:0010",
                    "run": "#decompress.tool.cwl",
                    "in": [
                        {
                            "source": "#main/mate_pair",
                            "valueFrom": "${ return self.reverse }\n",
                            "id": "#main/decompress_reverse/file"
                        },
                        {
                            "source": "#main/mate_pair",
                            "valueFrom": "${\n    if (!self.reverse.basename.match(/\\.tap\\./) ) {\n        var prefix = self.reverse.basename;\n        prefix = prefix.replace( /\\.fastq.gz$/ , '');\n        prefix = prefix.replace(/\\./ , \"_\") ;\n        return prefix + \".tap.\" + \"001\" + \".fastq\";\n    } else if (self.reverse.basename.match(/\\.gz$/) ) {\n        return self.reverse.basename.replace(/\\.gz$/ , '') ;\n    } else {\n        return self.reverse.basename ;\n    }\n }      \n",
                            "id": "#main/decompress_reverse/output"
                        }
                    ],
                    "out": [
                        "#main/decompress_reverse/decompressed"
                    ],
                    "id": "#main/decompress_reverse"
                },
                {
                    "run": {
                        "cwlVersion": "v1.0",
                        "class": "ExpressionTool",
                        "inputs": [
                            {
                                "type": "File",
                                "id": "#main/make_return/69e6d878-0c63-4518-b2b9-f40b941a6f21/forward"
                            },
                            {
                                "type": "File",
                                "id": "#main/make_return/69e6d878-0c63-4518-b2b9-f40b941a6f21/reverse"
                            }
                        ],
                        "outputs": [
                            {
                                "type": "Any",
                                "id": "#main/make_return/69e6d878-0c63-4518-b2b9-f40b941a6f21/mate_pair"
                            }
                        ],
                        "expression": "${\n  var mate_pair = {\n    \"forward\" : inputs.forward ,\n    \"reverse\" : inputs.reverse\n  };\n  return { 'mate_pair' : mate_pair} ;\n}\n",
                        "id": "#main/make_return/69e6d878-0c63-4518-b2b9-f40b941a6f21"
                    },
                    "in": [
                        {
                            "source": "#main/decompress_forward/decompressed",
                            "id": "#main/make_return/forward"
                        },
                        {
                            "source": "#main/decompress_reverse/decompressed",
                            "id": "#main/make_return/reverse"
                        }
                    ],
                    "out": [
                        "#main/make_return/mate_pair"
                    ],
                    "id": "#main/make_return"
                }
            ],
            "id": "#main"
        }
    ],
    "cwlVersion": "v1.0"
}
